import sqlite3 from 'sqlite3';

//TODO
// MUTEX functions for adding/removing/upsert
// basic read function by id
// complex read function by value range
class SqlSystem {
  private static db: sqlite3.Database;

  public static async InitTable() {
  const db = new sqlite3.Database('bot_data');
  try{
    await this.runQuery('CREATE TABLE IF NOT EXISTS users (id INTEGER PRIMARY KEY, name TEXT)')
    console.log('Table created successfully.');

    // Insert data
    await this.runQuery('INSERT INTO users (name) VALUES (?)', ['Alice']);
    console.log('Data inserted successfully.');

    // Retrieve a single row
    const row = await this.getQuery('SELECT * FROM users WHERE name = ?', ['Alice']);
    console.log('Retrieved row:', row);

    // Retrieve all rows
    const rows = await this.allQuery('SELECT * FROM users');
    console.log('All rows:', rows);
  } catch (err: any) {
    console.error('Database error:', err.message);
  } finally {
    db.close();
  }
}
  public static runQuery(query: string, params: string[] = []) {
  return new Promise((resolve,reject) => {
    this.db.run(query, params, function (err: Error | null | undefined) {
      if(err){
        reject(err);
      }
      else{
        resolve(this);
      }
    });
  });
}
  public static getQuery(query: string, params: string[] = []) {
  return new Promise((resolve, reject) => {
    this.db.get(query, params, (err: Error | null | undefined, row: sqlite3.RunResult) => {
      if (err) {
        reject(err);
      } else {
        resolve(row);
      }
    });
  });
}
  public static allQuery(query: string, params: string[] = []) {
  return new Promise((resolve, reject) => {
    this.db.all(query, params, (err: Error | null | undefined, rows: sqlite3.RunResult) => {
      if (err) {
        reject(err);
      } else {
        resolve(rows);
      }
    });
  });
}

}

export {
  SqlSystem
}